@@ -19,6 +19,10 @@
  *          Chris Phelps
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -26,7 +30,6 @@
 #include <dirent.h>
 #include <string.h>
 #include <time.h>
-#include <config.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <gdk/gdkx.h>
@@ -103,7 +106,7 @@ static const BonoboUIVerb applet_menu_verbs [] = {
 /* and the XML definition for the popup menu */
 static const char applet_menu_xml [] =
 "<popup name=\"button3\">\n"
-"   <menuitem name=\"RunGTCD\" verb=\"RunGTCD\" _label=\"Run GTCD...\"\n"
+"   <menuitem name=\"RunGTCD\" verb=\"RunGTCD\" _label=\"Run External CD Player\"\n"
 "             pixtype=\"stock\" pixname=\"gtk-cdrom\"/>\n"
 "   <menuitem name=\"Properties\" verb=\"Properties\" _label=\"Properties...\"\n"
 "             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
@@ -115,6 +118,7 @@ static const char applet_menu_xml [] =
 
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_CDPlayerApplet_Factory",
+			     PANEL_TYPE_APPLET,
                              "CD-Player-Applet",
                              "0",
                               applet_factory,
@@ -163,7 +167,7 @@ applet_fill (PanelApplet *applet)
     cd->panel.applet = GTK_WIDGET (applet);
     gtk_container_add (GTK_CONTAINER (applet), cdplayer);
 
-    panel_applet_add_preferences (applet, "/schemas/apps/cdplayer-applet/prefs", NULL);
+    /* panel_applet_add_preferences (applet, "/schemas/apps/cdplayer-applet/prefs", NULL); */
     cdplayer_load_config(cd);
 
     cd->cdrom_device = cdrom_open(cd->devpath, &err);
@@ -191,7 +195,7 @@ cdplayer_load_config(CDPlayerData *cd)
 {
     g_free(cd->devpath);
     cd->devpath = panel_applet_gconf_get_string(PANEL_APPLET(cd->panel.applet), "device-path", NULL);
-    if (!cd->devpath && !strcmp(cd->devpath, "none"))
+    if (!cd->devpath || !strcmp(cd->devpath, "none"))
     {
         g_free(cd->devpath);
         cd->devpath = g_strdup(DEV_PATH);
@@ -245,7 +249,50 @@ cdplayer_timeout_callback(gpointer data)
 static void
 start_gtcd_cb(GtkWidget *w, gpointer data)
 {
-    gnome_execute_shell(NULL, "gtcd");
+    gnome_execute_shell(NULL, "gnome-cd");
+}
+
+static void
+response_cb (GtkDialog *dialog, gint id, gpointer data)
+{
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+    
+}
+
+static void
+activate_cb (GtkEntry *entry, gpointer data)
+{
+    CDPlayerData *cd = data;
+    gchar *newpath, *oldpath;
+    
+    newpath = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+    if(newpath && strlen(newpath) > 2 && strcmp(cd->devpath, newpath))
+            {
+                cd_close(cd);
+                oldpath = cd->devpath;
+                cd->devpath = g_strdup(newpath);
+                if (cd_try_open(cd)) {
+                    cdplayer_save_config(cd);
+                    g_free (oldpath);
+                }
+                else {
+                    g_free (cd->devpath);
+                    cd->devpath = oldpath;
+                    cd_try_open (cd);
+                    gtk_entry_set_text (entry, cd->devpath);
+                }
+            }
+    if (newpath)
+        g_free (newpath);
+}
+
+static void
+focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+    CDPlayerData *cd = data;
+   
+    activate_cb (GTK_ENTRY (widget), cd);
+    
 }
 
 static void
@@ -255,18 +302,17 @@ properties_cb (GtkWidget *w, gpointer data)
     GtkWidget *dialog;
     GtkWidget *box;
     GtkWidget *hbox;
-    GtkWidget *frame;
-    GtkWidget *image;
     GtkWidget *label;
+    GtkWidget *image;
     GtkWidget *entry;
     gint response;
 
     cd = (CDPlayerData *) data;
 
-    dialog = gtk_dialog_new_with_buttons(_("CD Player Applet Properties"),
+    dialog = gtk_dialog_new_with_buttons(_("CD Player Properties"),
                                          NULL, GTK_DIALOG_MODAL,
                                          GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                                         GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+                                         NULL);
     box = GTK_DIALOG(dialog)->vbox;
 
     hbox = gtk_hbox_new(FALSE, 0);
@@ -277,24 +323,8 @@ properties_cb (GtkWidget *w, gpointer data)
     gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 10);
     gtk_widget_show(image);
 
-    label = gtk_label_new(_("NOTICE: These is a per-applet settings\n"
-                            "If you change them here, it will only change\n"
-                            "for this applet, allowing each instance to\n"
-                            "control a different cdrom device."));
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
-    gtk_widget_show(label);
-
-    frame = gtk_frame_new(_("Settings"));
-    gtk_box_pack_start(GTK_BOX(box), frame, TRUE, TRUE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
-    gtk_widget_show(frame);
-
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
-    gtk_container_add(GTK_CONTAINER(frame), hbox);
-    gtk_widget_show(hbox);
-
     label = gtk_label_new(_("Device Path:"));
+    gtk_misc_set_padding (GTK_MISC (label), GNOME_PAD_SMALL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
 
@@ -302,27 +332,16 @@ properties_cb (GtkWidget *w, gpointer data)
     gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
     gtk_widget_show(entry);
     gtk_entry_set_text(GTK_ENTRY(entry), cd->devpath);
+    g_signal_connect (G_OBJECT (entry), "activate",
+    		      G_CALLBACK (activate_cb), cd);
+    g_signal_connect (G_OBJECT (entry), "focus_out_event",
+    		      G_CALLBACK (focus_out_cb), cd);
+    
+    g_signal_connect (G_OBJECT (dialog), "response",
+    		      G_CALLBACK (response_cb), NULL);
 
     gtk_widget_show_all(dialog);
-    do
-    {
-        response = gtk_dialog_run(GTK_DIALOG(dialog));
-        if(response == GTK_RESPONSE_OK)
-        {
-            gchar *newpath;
-
-            newpath = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry));
-            if(newpath && strlen(newpath) > 2 && strcmp(cd->devpath, newpath))
-            {
-                cd_close(cd);
-                g_free(cd->devpath);
-                cd->devpath = g_strdup(newpath);
-                cd_try_open(cd);
-                cdplayer_save_config(cd);
-            }
-        }
-    } while(response == GTK_RESPONSE_APPLY);
-    gtk_widget_destroy(dialog);
+    
 }
 
 static void
