@@ -26,81 +26,59 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <X11/keysym.h>
 #include <X11/Xmd.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <sys/stat.h>
+
+#include <panel-applet.h>
+
+#include <gtk/gtk.h>
+#include <libbonobo.h>
+#include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
+#include <panel-applet-gconf.h>
 #include "gkb.h"
 
+int NumLockMask, CapsLockMask, ScrollLockMask;
+
 GtkWidget *bah_window = NULL;
 
-static void gkb_button_press_event_cb (GtkWidget * widget,
+gchar *
+gkb_load_pref(const gchar * key, const gchar * defaultv);
+
+static gint gkb_button_press_event_cb (GtkWidget * widget,
 				       GdkEventButton * e);
-static void about_cb (AppletWidget * widget);
-static void help_cb (AppletWidget * widget);
+static void about_cb (BonoboUIComponent *uic,
+			GKB	  *gkb,
+			const gchar	  *verbname);
+static void help_cb (BonoboUIComponent *uic,
+		        GKB       *gkb,
+	            const gchar	  *verbname);
 static GdkFilterReturn
 event_filter (GdkXEvent * gdk_xevent, GdkEvent * event, gpointer data);
 
+static void gkb_destroy (GtkWidget * widget, gpointer data);
+
 static void
-makepix (GkbKeymap * keymap, char *fname, int w, int h)
+set_tooltip (GtkWidget * w, const gchar * tip)
 {
-  GdkPixbuf *pix;
-  int width, height;
+  GtkTooltips *t = gtk_tooltips_new ();
+  gtk_tooltips_set_tip (t, w, tip, NULL);
+}
 
-  pix = gdk_pixbuf_new_from_file (fname);
-  if (pix != NULL)
-    {
-      double affine[6];
-      guchar *rgb;
-      GdkGC *gc;
-
-      g_assert (bah_window);
-      g_assert (bah_window->window);
-
-      width = gdk_pixbuf_get_width (pix);
-      height = gdk_pixbuf_get_height (pix);
-
-      affine[1] = affine[2] = affine[4] = affine[5] = 0;
-
-      affine[0] = w / (double) width;
-      affine[3] = h / (double) height;
-
-      rgb = g_new0 (guchar, w * h * 3);
-
-      art_rgb_rgba_affine (rgb,
-			   0, 0, w, h, w * 3,
-			   gdk_pixbuf_get_pixels (pix),
-			   width, height,
-			   gdk_pixbuf_get_rowstride (pix),
-			   affine, ART_FILTER_NEAREST, NULL);
-
-      gdk_pixbuf_unref (pix);
-
-      keymap->pix = gdk_pixmap_new (bah_window->window, w, h,
-#if 0
-				    gtk_widget_get_visual (gkb->darea)->depth
-#else
-				    -1
-#endif
-	);
-
-      gc = gdk_gc_new (keymap->pix);
-      gdk_draw_rgb_image (keymap->pix, gc,
-			  0, 0, w, h, GDK_RGB_DITHER_NORMAL, rgb, w * 3);
-      gdk_gc_destroy (gc);
-
-      g_free (rgb);
-    }
-  else
-    {
-      if (keymap->pix)
-	{
-	  gdk_pixmap_unref (keymap->pix);
-	}
-      keymap->pix = gdk_pixmap_new (bah_window->window, w, h, -1);
-    }
+void alert(const gchar * str){
+ GtkWidget * window;
+ window = gnome_message_box_new (str,
+               GNOME_MESSAGE_BOX_INFO,
+               GNOME_STOCK_BUTTON_OK, NULL);
+ gtk_widget_show_all(window);
 }
 
 /**
@@ -112,22 +90,23 @@ makepix (GkbKeymap * keymap, char *fname, int w, int h)
 static void
 gkb_draw (GKB * gkb)
 {
-  g_return_if_fail (gkb->darea != NULL);
+  g_return_if_fail (gkb->image != NULL);
   g_return_if_fail (gkb->keymap != NULL);
-  g_return_if_fail (GTK_WIDGET_REALIZED (gkb->darea));
 
   if (gkb->mode != GKB_LABEL)
     {
+      GdkPixbuf *tmp;
+      
       g_return_if_fail (gkb != NULL);
       g_return_if_fail (gkb->keymap != NULL);
-      g_return_if_fail (gkb->keymap->pix != NULL);
-      g_return_if_fail (gkb->darea);
-      g_return_if_fail (gkb->darea->window);
-
-      gdk_draw_pixmap (gkb->darea->window,
-		       gkb->darea->
-		       style->fg_gc[GTK_WIDGET_STATE (gkb->darea)],
-		       gkb->keymap->pix, 0, 0, 0, 0, gkb->w, gkb->h);
+      g_return_if_fail (gkb->keymap->pixbuf != NULL);
+      
+      tmp = gdk_pixbuf_scale_simple (gkb->keymap->pixbuf, gkb->w, 
+      				     gkb->h, GDK_INTERP_HYPER); 
+      if (tmp) {     
+        gtk_image_set_from_pixbuf (GTK_IMAGE (gkb->image), tmp);
+        g_object_unref (tmp);
+      }
 
     }
 
@@ -137,7 +116,8 @@ gkb_draw (GKB * gkb)
       gtk_label_set_text (GTK_LABEL (gkb->label2), gkb->keymap->label);
     }
 
-  applet_widget_set_tooltip (APPLET_WIDGET (gkb->applet), gkb->keymap->name);
+  set_tooltip (gkb->applet, gkb->keymap->name);
+
 }
 
 
@@ -164,19 +144,19 @@ gkb_count_sizes (GKB * gkb)
 
   gint size;
 
-  size = applet_widget_get_panel_pixel_size (APPLET_WIDGET (gkb->applet));
+  size = panel_applet_get_size (PANEL_APPLET (gkb->applet));
 
   /* Determine if this pannel requires different handling because it is very small */
   switch (gkb->orient)
     {
-    case ORIENT_UP:
-    case ORIENT_DOWN:
+    case PANEL_APPLET_ORIENT_UP:
+    case PANEL_APPLET_ORIENT_DOWN:
       panel_height = size;
       if (size < GKB_SMALL_PANEL_SIZE)
 	small_panel = TRUE;
       break;
-    case ORIENT_RIGHT:
-    case ORIENT_LEFT:
+    case PANEL_APPLET_ORIENT_RIGHT:
+    case PANEL_APPLET_ORIENT_LEFT:
       panel_width = size;
       if (size < GKB_SMALL_PANEL_SIZE)
 	small_panel = TRUE;
@@ -193,18 +173,18 @@ gkb_count_sizes (GKB * gkb)
    * v.s. beeing top-bottom */
   if (gkb->mode == GKB_FLAG_AND_LABEL)
     {
-      if (gkb->orient == ORIENT_UP && small_panel)
+      if (gkb->orient == PANEL_APPLET_ORIENT_UP && small_panel)
 	label_in_vbox = FALSE;
-      if (gkb->orient == ORIENT_UP && !gkb->is_small)
+      if (gkb->orient == PANEL_APPLET_ORIENT_UP && !gkb->is_small)
 	label_in_vbox = FALSE;
-      if (gkb->orient == ORIENT_DOWN && small_panel)
+      if (gkb->orient == PANEL_APPLET_ORIENT_DOWN && small_panel)
 	label_in_vbox = FALSE;
-      if (gkb->orient == ORIENT_DOWN && !gkb->is_small)
+      if (gkb->orient == PANEL_APPLET_ORIENT_DOWN && !gkb->is_small)
 	label_in_vbox = FALSE;
-      if (gkb->orient == ORIENT_RIGHT && !small_panel)
+      if (gkb->orient == PANEL_APPLET_ORIENT_RIGHT && !small_panel)
 	if (gkb->is_small)
 	  label_in_vbox = FALSE;
-      if (gkb->orient == ORIENT_LEFT && !small_panel)
+      if (gkb->orient == PANEL_APPLET_ORIENT_LEFT && !small_panel)
 	if (gkb->is_small)
 	  label_in_vbox = FALSE;
     }
@@ -232,23 +212,12 @@ gkb_count_sizes (GKB * gkb)
 	applet_width += label_width;
     }
 
-  gtk_widget_set_usize (GTK_WIDGET (gkb->applet), applet_width,
-			applet_height);
-
-  if (flag_width > 0)
-    {
-#if 1
-      gtk_widget_set_usize (GTK_WIDGET (gkb->darea), flag_width, flag_height);
-#endif
-      gtk_drawing_area_size (GTK_DRAWING_AREA (gkb->darea), flag_width,
-			     flag_height);
-    }
-
-  gtk_widget_set_usize (GTK_WIDGET (gkb->label1), label_width, label_height);
-  gtk_widget_set_usize (GTK_WIDGET (gkb->label2), label_width, label_height);
-
   gkb->w = flag_width;
-  gkb->h = flag_height;
+  /* FIXME the applet is just a little bigger than panel, so I add the -1 here*/
+  gkb->h = flag_height - 1; 
+  gtk_widget_set_size_request (gkb->label_frame1, gkb->w, gkb->h);
+  gtk_widget_set_size_request (gkb->label_frame2, gkb->w, gkb->h);
+  gtk_widget_set_size_request (gkb->darea_frame, gkb->w, gkb->h);
 
   return label_in_vbox;
 }
@@ -268,7 +237,7 @@ gkb_sized_render (GKB * gkb)
   gboolean label_in_vbox;
 
   label_in_vbox = gkb_count_sizes (gkb);
-
+ 
   /* Hide or show the flag */
   if (gkb->mode == GKB_LABEL)
     gtk_widget_hide (gkb->darea_frame);
@@ -306,23 +275,18 @@ gkb_sized_render (GKB * gkb)
       real_name = gnome_unconditional_pixmap_file (name);
       if (g_file_exists (real_name))
 	{
-	  makepix (keymap, real_name, gkb->w - 4, gkb->h - 4);
+	  keymap->pixbuf = gdk_pixbuf_new_from_file (real_name,NULL);
 	}
       else
 	{
 	  g_free (real_name);
 	  real_name = gnome_unconditional_pixmap_file ("gkb/gkb-foot.png");
-	  makepix (keymap, real_name, gkb->w - 4, gkb->h - 4);
+	  keymap->pixbuf = gdk_pixbuf_new_from_file (real_name,NULL);
 	}
       g_free (name);
       g_free (real_name);
     }
 
-#if 0
-  gtk_widget_queue_resize (gkb->darea);
-  gtk_widget_queue_resize (gkb->darea->parent);
-  gtk_widget_queue_resize (gkb->darea->parent->parent);
-#endif
 }
 
 /**
@@ -350,7 +314,7 @@ gkb_update (GKB * gkb, gboolean set_command)
 }
 
 static void
-gkb_change_orient (GtkWidget * w, PanelOrientType new_orient, gpointer data)
+gkb_change_orient (GtkWidget * w, PanelAppletOrient new_orient, gpointer data)
 {
   GKB *gkb = data;
 
@@ -377,9 +341,8 @@ gkb_change_pixel_size (GtkWidget * w, gint new_size, gpointer data)
 #endif
 }
 
-static gboolean
-applet_save_session (GtkWidget * w,
-		     const char *privcfgpath, const char *globcfgpath)
+void
+applet_save_session ()
 {
   const gchar *text;
   GkbKeymap *actdata;
@@ -387,43 +350,48 @@ applet_save_session (GtkWidget * w,
   gchar str[100];
   int i = 0;
 
-  gnome_config_push_prefix (privcfgpath);
-  g_print ("Pushing prefix .. [%s]\n", privcfgpath);
-  gnome_config_set_int ("gkb/num", gkb->n);
-  gnome_config_set_bool ("gkb/small", gkb->is_small);
-  gnome_config_set_string ("gkb/key", gkb->key);
+  panel_applet_gconf_set_int (PANEL_APPLET (gkb->applet), "num", gkb->n, NULL);
+  panel_applet_gconf_set_bool (PANEL_APPLET (gkb->applet), "small", gkb->is_small, NULL);
+  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), "key", gkb->key, NULL);
   text = gkb_util_get_text_from_mode (gkb->mode);
-  gnome_config_set_string ("gkb/mode", text);
+  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), "mode", text, NULL);
 
   while (list)
     {
       actdata = list->data;
       if (actdata)
 	{
-	  g_snprintf (str, sizeof (str), "keymap_%d/name", i);
-	  gnome_config_set_string (str, actdata->name);
-	  g_snprintf (str, sizeof (str), "keymap_%d/country", i);
-	  gnome_config_set_string (str, actdata->country);
-	  g_snprintf (str, sizeof (str), "keymap_%d/lang", i);
-	  gnome_config_set_string (str, actdata->lang);
-	  g_snprintf (str, sizeof (str), "keymap_%d/label", i);
-	  gnome_config_set_string (str, actdata->label);
-	  g_snprintf (str, sizeof (str), "keymap_%d/flag", i);
-	  gnome_config_set_string (str, actdata->flag);
-	  g_snprintf (str, sizeof (str), "keymap_%d/command", i);
-	  gnome_config_set_string (str, actdata->command);
+	  g_snprintf (str, sizeof (str), "name_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->name, NULL);
+	  g_snprintf (str, sizeof (str), "country_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->country, NULL);
+	  g_snprintf (str, sizeof (str), "lang_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->lang, NULL);
+	  g_snprintf (str, sizeof (str), "label_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->label, NULL);
+	  g_snprintf (str, sizeof (str), "flag_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->flag, NULL);
+	  g_snprintf (str, sizeof (str), "command_%d", i);
+	  panel_applet_gconf_set_string (PANEL_APPLET (gkb->applet), str, actdata->command, NULL);
 	}
 
       list = list->next;
 
       i++;
     }
+}
 
-  gnome_config_pop_prefix ();
-  gnome_config_sync ();
-  gnome_config_drop_all ();
+gchar *
+gkb_load_pref(const gchar * key, const gchar * defaultv)
+{
+ gchar * value;
 
-  return FALSE;
+ value = panel_applet_gconf_get_string (PANEL_APPLET(gkb->applet), key, NULL);
+ 
+ if (value == NULL) {
+  value = g_strdup (defaultv);
+ }
+ return value;
 }
 
 GkbKeymap *
@@ -434,51 +402,28 @@ loadprop (int i)
 
   actdata = g_new0 (GkbKeymap, 1);
 
-  if (i == 0)
-    {
-      buf = g_strdup_printf (_("keymap_%d/name=US 105 key keyboard (with windows keys)"),i);
-      actdata->name = gnome_config_get_string (buf);
+  buf = g_strdup_printf (_("name_%d"),i);
+  actdata->name = gkb_load_pref (buf, (i?"Hungarian 105 keys keyboard":"US 105 key keyboard"));
 
-      buf = g_strdup_printf (_("keymap_%d/label=us"), i);
-      actdata->label = gnome_config_get_string (buf);
+  buf = g_strdup_printf (_("label_%d"), i);
+  actdata->label = gkb_load_pref (buf, (i?"hu":"us"));
 
-      buf = g_strdup_printf (_("keymap_%d/country=United States"), i);
-      actdata->country = gnome_config_get_string (buf);
+  buf = g_strdup_printf (_("country_%d"), i);
+  actdata->country = gkb_load_pref (buf, (i?"Hungary":"United States"));
 
-      buf = g_strdup_printf (_("keymap_%d/lang=English"), i);
-      actdata->lang = gnome_config_get_string (buf);
+  buf = g_strdup_printf (_("lang_%d"), i);
+  actdata->lang = gkb_load_pref (buf, (i?"Hungarian":"English"));
 
-      buf = g_strdup_printf (("keymap_%d/flag=us.png"), i);
-      actdata->flag = gnome_config_get_string (buf);
+  buf = g_strdup_printf (("flag_%d"), i);
+  actdata->flag = gkb_load_pref (buf, (i?"hu.png":"us.png"));
 
-      buf = g_strdup_printf (_("keymap_%d/command=gkb_xmmap us"), i);
-      actdata->command = gnome_config_get_string (buf);
-      g_free(buf);
-    }
-  else
-    {
-      buf = g_strdup_printf (_("keymap_%d/name=Hungarian 105 keys latin2"), i);
-      actdata->name = gnome_config_get_string (buf);
+  buf = g_strdup_printf (_("command_%d"), i);
+  actdata->command = gkb_load_pref (buf, (i?"gkb_xmmap hu":"gkb_xmmap us"));
 
-      buf = g_strdup_printf (_("keymap_%d/label=hu"), i);
-      actdata->label = gnome_config_get_string (buf);
+  g_free(buf);
 
-      buf = g_strdup_printf (_("keymap_%d/country=Hungary"), i);
-      actdata->country = gnome_config_get_string (buf);
-
-      buf = g_strdup_printf (_("keymap_%d/lang=Hungarian"), i);
-      actdata->lang = gnome_config_get_string (buf);
-
-      buf = g_strdup_printf (_("keymap_%d/flag=hu.png"), i);
-      actdata->flag = gnome_config_get_string (buf);
-
-      buf = g_strdup_printf (_("keymap_%d/command=gkb_xmmap hu"), i);
-      actdata->command = gnome_config_get_string (buf);
-      g_free(buf);
-    }
-
-  actdata->pix = NULL;
-  gkb->orient = applet_widget_get_panel_orient (APPLET_WIDGET (gkb->applet));
+  actdata->pixbuf = NULL;
+  gkb->orient = panel_applet_get_orient (PANEL_APPLET (gkb->applet));
 
   return actdata;
 }
@@ -492,16 +437,18 @@ load_properties (GKB * gkb)
 
   gkb->maps = NULL;
 
-  gnome_config_push_prefix (APPLET_WIDGET (gkb->applet)->privcfgpath);
+  gkb->n = panel_applet_gconf_get_int (PANEL_APPLET(gkb->applet), "num", NULL);
 
-  gkb->n = gnome_config_get_int ("gkb/num=0");
-
-  gkb->key = gnome_config_get_string ("gkb/key=Mod1-Shift_L");
+  gkb->key = gkb_load_pref ("key", "Mod1-Shift_L");
   convert_string_to_keysym_state (gkb->key, &gkb->keysym, &gkb->state);
 
-  gkb->is_small = gnome_config_get_bool ("gkb/small=true");
-
-  text = gnome_config_get_string ("gkb/mode=Flag and Label");
+  gkb->is_small = panel_applet_gconf_get_bool (PANEL_APPLET(gkb->applet), "small", NULL);
+#ifdef THIS_IS_UNNECESSARY
+  if (gkb->is_small == NULL) {
+    gkb->is_small = TRUE;
+  }
+#endif 
+  text = gkb_load_pref ("mode", "Flag and Label");
   gkb->mode = gkb_util_get_mode_from_text (text);
   g_free (text);
 
@@ -522,18 +469,16 @@ load_properties (GKB * gkb)
 	gkb->maps = g_list_append (gkb->maps, actdata);
       }
 
-  gnome_config_pop_prefix ();
-
-  /* tell the panel to save our configuration data */
-  applet_widget_sync_config (APPLET_WIDGET (gkb->applet));
+  applet_save_session ();
 }
 
 
-static void
+static gint
 gkb_button_press_event_cb (GtkWidget * widget, GdkEventButton * event)
 {
-  if (event->button != 1)	/* Ignore mouse buttons 2 and 3 */
-    return;
+   if (event->button != 1)	
+    return FALSE;
+
 
   if (gkb->cur + 1 < gkb->n)
     gkb->keymap = g_list_nth_data (gkb->maps, ++gkb->cur);
@@ -544,65 +489,31 @@ gkb_button_press_event_cb (GtkWidget * widget, GdkEventButton * event)
     }
 
   gkb_update (gkb, TRUE);
-}
-
-static int
-gkb_expose (GtkWidget * darea, GdkEventExpose * event)
-{
-  gdk_draw_pixmap (gkb->darea->window,
-		   gkb->darea->style->fg_gc[GTK_WIDGET_STATE (gkb->darea)],
-		   gkb->keymap->pix,
-		   event->area.x, event->area.y,
-		   event->area.x, event->area.y,
-		   event->area.width, event->area.height);
-
-  return FALSE;
+  return TRUE;
 }
 
 static void
 create_gkb_widget ()
 {
-  GtkStyle *style;
-
-  gtk_widget_push_visual (gdk_rgb_get_visual ());
-  gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-
-  style = gtk_widget_get_style (gkb->applet);
-
   gkb->eventbox = gtk_event_box_new ();
   gtk_widget_show (gkb->eventbox);
 
-  gkb->vbox = gtk_vbox_new (FALSE, 0);
+  gkb->vbox = gtk_vbox_new (TRUE, 0);
   gtk_widget_show (gkb->vbox);
-  gkb->hbox = gtk_hbox_new (FALSE, 0);
+  gkb->hbox = gtk_hbox_new (TRUE, 0);
   gtk_widget_show (gkb->hbox);
 
   gtk_container_add (GTK_CONTAINER (gkb->eventbox), gkb->hbox);
   gtk_container_add (GTK_CONTAINER (gkb->hbox), gkb->vbox);
 
-  gkb->darea = gtk_drawing_area_new ();
+  gkb->image = gtk_image_new ();
 
-  gtk_drawing_area_size (GTK_DRAWING_AREA (gkb->darea), gkb->w, gkb->h);
-
-  gtk_widget_set_events (gkb->darea,
-			 gtk_widget_get_events (gkb->darea) |
-			 GDK_BUTTON_PRESS_MASK);
-
-  gtk_signal_connect (GTK_OBJECT (gkb->eventbox), "button_press_event",
-		      GTK_SIGNAL_FUNC (gkb_button_press_event_cb), NULL);
-#if 0
-  gtk_signal_connect (GTK_OBJECT (gkb->eventbox), "expose_event",
-		      GTK_SIGNAL_FUNC (gkb_expose), NULL);
-#else
-  gtk_signal_connect (GTK_OBJECT (gkb->darea), "expose_event",
-		      GTK_SIGNAL_FUNC (gkb_expose), NULL);
-#endif
-
-  gtk_widget_show (gkb->darea);
+  g_signal_connect (gkb->eventbox, "button_press_event",
+                    G_CALLBACK (gkb_button_press_event_cb), NULL);
 
   gkb->darea_frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (gkb->darea_frame), GTK_SHADOW_IN);
-  gtk_container_add (GTK_CONTAINER (gkb->darea_frame), gkb->darea);
+  gtk_container_add (GTK_CONTAINER (gkb->darea_frame), gkb->image);
   gtk_box_pack_start (GTK_BOX (gkb->vbox), gkb->darea_frame, TRUE, TRUE, 0);
 #if 0
   g_print ("c\n");
@@ -624,66 +535,87 @@ create_gkb_widget ()
   gtk_container_add (GTK_CONTAINER (gkb->label_frame2), gkb->label2);
   gtk_container_add (GTK_CONTAINER (gkb->hbox), gkb->label_frame2);
 
-  gtk_widget_pop_colormap ();
-  gtk_widget_pop_visual ();
-
-  gkb_sized_render (gkb);
-  gkb_update (gkb, TRUE);
 }
 
 static void
-about_cb (AppletWidget * widget)
+about_cb (BonoboUIComponent *uic,
+          GKB     *gkb,
+          const gchar     *verbname)
 {
   static GtkWidget *about;
-  const char *authors[3];	/* Emese is genius */
   GtkWidget *link;
+  gchar *file;	
+  GdkPixbuf *pixbuf;
 
-/*
-  if (about)
-    {
-      gtk_widget_show (about);
-      gdk_window_raise (about->window);
-      return;
-    }
-*/
+  static const gchar *authors[] = {
+		"Chema Celorio <chema@celorio.com>",
+	        "Szabolcs Ban <shooby@gnome.hu>",
+		NULL
+	};
+  static const gchar *docauthors[] = {
+	        "Szabolcs Ban <shooby@gnome.hu>",
+		"Emese Kovacs <emese@gnome.hu>",
+		"David Mason <dcm@redhat.com>",
+		"Alexander Kirillov <kirillov@math.sunysb.edu>",
+		NULL
+	};
+	
 
-  authors[0] = N_("Szabolcs BAN <shooby@gnome.hu>");
-  authors[1] = N_("Chema Celorio <chema@celorio.com>");
-  authors[2] = NULL;
+  pixbuf = NULL;
+  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gkb-icon.png", TRUE, NULL);
+  if (!file) {
+         g_warning (G_STRLOC ": gkb-icon.png cannot be found");
+         pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+  }
 
   about = gnome_about_new (_("The GNOME KeyBoard Switcher Applet"),
 			   VERSION,
-			   _
-			   ("(C) 1998-2000 Free Software Foundation"),
-			   authors,
-			   _
-			   ("This applet switches between keyboard maps. "
+			   _("(C) 1998-2000 Free Software Foundation"),
+ 			   _("This applet switches between keyboard maps. "
 			    "It uses setxkbmap, or xmodmap.\n"
 			    "Mail me your flag, please (60x40 size), "
-			    "I will put it to CVS.\n"
+			    "I will put it into CVS.\n"
 			    "So long, and thanks for all the fish.\n"
 			    "Thanks for Balazs Nagy (Kevin) "
-			    "<julian7@kva.hu> for his help "
-			    "and Emese Kovacs <emese@eik.bme.hu> for "
-			    "her solidarity."), "gkb-icon.png");
+			    "<julian7@iksz.hu> for his help "
+			    "and Emese Kovacs <emese@gnome.hu> for "
+			    "her solidarity."),
+			    authors,
+			    docauthors,
+			    NULL,
+			    pixbuf);
 
   link = gnome_href_new ("http://projects.gnome.hu/gkb",
 			 _("GKB Home Page (http://projects.gnome.hu/gkb)"));
+
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox), link, TRUE,
 		      FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (about), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_widget_destroyed), &about);
+
+  gtk_window_set_wmclass (GTK_WINDOW (about), "gkb", "Gkb");
+
+  if (pixbuf) {
+                gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
+                g_object_unref (pixbuf);
+  }
+
+  g_signal_connect (G_OBJECT(about), "destroy",
+                          (GCallback)gtk_widget_destroyed, &about);
+
   gtk_widget_show_all (about);
 
   return;
 }
 
 static void
-help_cb (AppletWidget * applet)
+help_cb (BonoboUIComponent *uic,
+         GKB	 *gkb,
+         const gchar	 *verbname)
 {
+/* TODO:
   GnomeHelpMenuEntry help_entry = { "gkb_applet", "index.html" };
 
   gnome_help_display (NULL, &help_entry);
+*/
 }
 
 static GdkFilterReturn
@@ -720,56 +652,6 @@ event_filter (GdkXEvent * gdk_xevent, GdkEvent * event, gpointer data)
   return GDK_FILTER_CONTINUE;
 }
 
-/**
- * gkb_activator_connect_signals:
- * @gkb: 
- * 
- * Connect the signaals for gkb->applet
- **/
-static void
-gkb_activator_connect_signals (GKB * gkb)
-{
-  g_return_if_fail (gkb != NULL);
-  g_return_if_fail (GTK_IS_WIDGET (gkb->applet));
-
-  gtk_signal_connect (GTK_OBJECT (gkb->applet), "change_orient",
-		      GTK_SIGNAL_FUNC (gkb_change_orient), gkb);
-
-  gtk_signal_connect (GTK_OBJECT (gkb->applet), "change_pixel_size",
-		      GTK_SIGNAL_FUNC (gkb_change_pixel_size), gkb);
-}
-
-/**
- * gkb_applet_widget_register_callbacks:
- * @gkb: 
- * 
- * Register the callbacks for the applet widget
- **/
-static void
-gkb_activator_register_callbacks (GKB * gkb)
-{
-  g_return_if_fail (gkb != NULL);
-  g_return_if_fail (GTK_IS_WIDGET (gkb->applet));
-
-  applet_widget_register_stock_callback (APPLET_WIDGET (gkb->applet),
-					 "properties",
-					 GNOME_STOCK_MENU_PROP,
-					 _("Properties..."),
-					 GTK_SIGNAL_FUNC
-					 (properties_dialog), NULL);
-  applet_widget_register_stock_callback (APPLET_WIDGET (gkb->applet),
-					 "help",
-					 GNOME_STOCK_MENU_BOOK_OPEN,
-					 _("Help"),
-					 GTK_SIGNAL_FUNC (help_cb), NULL);
-  applet_widget_register_stock_callback (APPLET_WIDGET (gkb->applet),
-					 "about",
-					 GNOME_STOCK_MENU_ABOUT,
-					 _("About..."),
-					 GTK_SIGNAL_FUNC (about_cb), NULL);
-
-}
-
 static gboolean
 check_client (Display * display, Window xwindow, Atom state_atom)
 {
@@ -840,111 +722,104 @@ find_input_client (Display * display, Window xwindow, Atom state_atom)
   return xwindow;
 }
 
-static CORBA_Object
-gkb_activator (CORBA_Object poa_in,
-	       const char *goad_id,
-	       const char **params,
-	       gpointer * impl_ptr, CORBA_Environment * ev)
-{
-  static guint key = 0;
-  PortableServer_POA poa = (PortableServer_POA) poa_in;
-  XWindowAttributes attribs = { 0, };
+static const char gkb_menu_xml [] =
+	"<popup name=\"button3\">\n"
+	"   <menuitem name=\"GKB Properties Item\" verb=\"GKBProperties\" _label=\"Properties ...\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
+	"   <menuitem name=\"GKB Help Item\" verb=\"GKBHelp\" _label=\"Help\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
+	"   <menuitem name=\"GKB About Item\" verb=\"GKBAbout\" _label=\"About ...\"\n"
+	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
+	"</popup>\n";
 
+static const BonoboUIVerb gkb_menu_verbs [] = {
+	BONOBO_UI_UNSAFE_VERB ("GKBProperties",   properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("GKBHelp",	  help_cb),
+	BONOBO_UI_UNSAFE_VERB ("GKBAbout",	  about_cb),
+	BONOBO_UI_VERB_END
+};
+
+static void gkb_destroy (GtkWidget * widget, gpointer data) {
+	g_free(gkb);
+}
+
+gboolean fill_gkb_applet(PanelApplet *applet)
+{
+  gint keycode, modifiers;
   gkb = g_new0 (GKB, 1);
 
   gkb->n = 0;
   gkb->cur = 0;
-  gkb->applet = applet_widget_new (goad_id);
-
-  gtk_widget_push_visual (gdk_rgb_get_visual ());
-  gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+  gkb->applet = GTK_WIDGET (applet);
 
+  panel_applet_add_preferences (applet, "/schemas/apps/gkb-applet/prefs", NULL);
+  /* FIXME: is bah_window needed for anything */
   bah_window = gtk_window_new (GTK_WINDOW_POPUP);
 
-  gtk_widget_pop_visual ();
-  gtk_widget_pop_colormap ();
-
   gtk_widget_set_uposition (bah_window, gdk_screen_width () + 1,
 			    gdk_screen_height () + 1);
-  gtk_widget_show_now (bah_window);
+  gtk_widget_show (bah_window);
 
   load_properties (gkb);
 
-  gkb_activator_connect_signals (gkb);
-
   gkb->keymap = g_list_nth_data (gkb->maps, 0);
 
   create_gkb_widget ();
 
   gtk_widget_show (gkb->darea_frame);
-  applet_widget_add (APPLET_WIDGET (gkb->applet), gkb->eventbox);
-  gtk_widget_show (gkb->applet);
+  gtk_container_add (GTK_CONTAINER (gkb->applet), gkb->eventbox);
 
-  gtk_signal_connect (GTK_OBJECT (gkb->applet), "save_session",
-		      GTK_SIGNAL_FUNC (applet_save_session), NULL);
+  keycode = XKeysymToKeycode(GDK_DISPLAY(), gkb->keysym);
 
-  XGetWindowAttributes (GDK_DISPLAY (), GDK_ROOT_WINDOW (), &attribs);
-  XSelectInput (GDK_DISPLAY (),
-		GDK_ROOT_WINDOW (),
-		attribs.your_event_mask |
-		StructureNotifyMask | FocusChangeMask | PropertyChangeMask);
+  modifiers = gkb->state;
+  
+  gkb_xgrab (keycode, modifiers);
 
   gdk_window_add_filter (GDK_ROOT_PARENT (), event_filter, NULL);
 
-  key = XKeysymToKeycode (GDK_DISPLAY (), gkb->keysym);
+  g_signal_connect (G_OBJECT(gkb->applet),
+  			  "destroy", 
+  			  G_CALLBACK (gkb_destroy), 
+  			  NULL);
 
-  XGrabKey (GDK_DISPLAY (), key, gkb->state,
-	    GDK_ROOT_WINDOW (), True, GrabModeAsync, GrabModeAsync);
+  g_signal_connect (G_OBJECT (gkb->applet),
+                          "change_orient",
+                          G_CALLBACK (gkb_change_orient),
+                          gkb);
 
-  gkb_activator_register_callbacks (gkb);
+  g_signal_connect (G_OBJECT (gkb->applet),
+                          "change_size",
+                          G_CALLBACK (gkb_change_pixel_size),
+                          gkb);
 
-  return applet_widget_corba_activate (gkb->applet, poa, goad_id,
-				       params, impl_ptr, ev);
+  panel_applet_setup_menu (PANEL_APPLET (gkb->applet), 
+  				gkb_menu_xml, 
+  				gkb_menu_verbs, 
+  				gkb);
+
+  gtk_widget_show (GTK_WIDGET(gkb->applet));
+  gkb_sized_render (gkb);
+  gkb_update(gkb,TRUE);
+
+  return TRUE;
 }
 
-static void
-gkb_deactivator (CORBA_Object poa_in,
-		 const char *goad_id,
-		 gpointer impl_ptr, CORBA_Environment * ev)
+static gboolean
+gkb_factory (PanelApplet *applet,
+		const gchar *iid,
+		gpointer     data)
 {
-  PortableServer_POA poa = (PortableServer_POA) poa_in;
+ 	gboolean retval = FALSE;
 
-  gdk_window_remove_filter (GDK_ROOT_PARENT (), event_filter, NULL);
+        if (!strcmp (iid, "OAFIID:GNOME_KeyboardApplet"))
+		retval = fill_gkb_applet (applet);
 
-  applet_widget_corba_deactivate (poa, goad_id, impl_ptr, ev);
+	return retval;
 }
 
-#if 0 || defined(SHLIB_APPLETS)
-static const char *repo_id[] = { "IDL:GNOME/Applet:1.0", NULL };
-static GnomePluginObject applets_list[] = {
-  {repo_id, "gkb_applet", NULL, "GNOME Keyboard switcher",
-   &gkb_activator, &gkb_deactivator},
-  {NULL}
-};
-
-GnomePlugin GNOME_Plugin_info = {
-  applets_list, NULL
-};
-#else
-
-int
-main (int argc, char *argv[])
-{
-  gpointer gkb_impl;
-
-  /* Initialize the i18n stuff */
-
-  bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-  textdomain (PACKAGE);
-
-  applet_widget_init ("gkb_applet", VERSION, argc, argv, NULL, 0, NULL);
-
-  APPLET_ACTIVATE (gkb_activator, "gkb_applet", &gkb_impl);
-
-  applet_widget_gtk_main ();
-
-  APPLET_DEACTIVATE (gkb_deactivator, "gkb_applet", gkb_impl);
-
-  return 0;
-}
-#endif
+PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_KeyboardApplet_Factory",
+			     PANEL_TYPE_APPLET,
+                             "GKB_keyboard_layout_switcher",
+                             "0",
+                             gkb_factory,
+                             NULL)
