@@ -102,19 +102,12 @@ typedef struct {
 	GtkAdjustment     *adj;
 
 	GtkWidget         *applet;
-	GtkWidget         *event_box;
 	GtkWidget         *frame;
 	GtkWidget         *image;
 	
 	/* The popup window and scale. */
 	GtkWidget         *popup;
 	GtkWidget         *scale;
-
-	GdkPixbuf         *zero_pixbuf;
-	GdkPixbuf         *min_pixbuf;
-	GdkPixbuf         *medium_pixbuf;
-	GdkPixbuf         *max_pixbuf;
-	GdkPixbuf         *mute_pixbuf;
 } MixerData;
 
 static void mixer_update_slider (MixerData *data);
@@ -122,8 +115,15 @@ static void mixer_update_image  (MixerData *data);
 static void mixer_popup_show    (MixerData *data);
 static void mixer_popup_hide    (MixerData *data, gboolean revert);
 
+static void mixer_start_gmix_cb (BonoboUIComponent *uic,
+				 gpointer           data,
+				 const gchar       *verbname);
+
+
+static gint mixerfd = -1;
+static gint num_applets = 0;
+static GdkPixbuf *zero_pixbuf, *min_pixbuf, *medium_pixbuf, *max_pixbuf, *mute_pixbuf;
 
-static gint       mixerfd = -1;
 
 #ifdef OSS_API
 static int mixerchannel;
@@ -278,16 +278,12 @@ mixer_value_changed_cb (GtkAdjustment *adj, MixerData *data)
 {
 	gint vol;
 
-	if (IS_PANEL_HORIZONTAL (data->orientation)) {
-		vol = -GTK_ADJUSTMENT (adj)->value;
-	} else {
-		vol = -(-VOLUME_MAX - GTK_ADJUSTMENT (adj)->value);
-	}
-
+	vol = -adj->value;
+	
 	data->vol = vol;
 
 	mixer_update_image (data);
-	
+
 	if (!data->mute) {
 		setMixer (vol);
 	}
@@ -346,16 +342,16 @@ mixer_update_image (MixerData *data)
 	vol = data->vol;
 	
 	if (vol <= 0) {
-		pixbuf = data->zero_pixbuf;
+		pixbuf = zero_pixbuf;
 	}
 	else if (vol <= VOLUME_MAX / 3) {
-		pixbuf = data->min_pixbuf;
+		pixbuf = min_pixbuf;
 	}
 	else if (vol <= 2 * VOLUME_MAX / 3) {
-		pixbuf = data->medium_pixbuf;
+		pixbuf = medium_pixbuf;
 	}
 	else {
-		pixbuf = data->max_pixbuf;
+		pixbuf = max_pixbuf;
 	}
 
 	if (!data->mute) {
@@ -363,12 +359,12 @@ mixer_update_image (MixerData *data)
 	} else {
 		GdkPixbuf *copy = gdk_pixbuf_copy (pixbuf);
 
-		gdk_pixbuf_composite (data->mute_pixbuf,
+		gdk_pixbuf_composite (mute_pixbuf,
 				      copy,
 				      0,
 				      0,
-				      gdk_pixbuf_get_width (data->mute_pixbuf),
-				      gdk_pixbuf_get_height (data->mute_pixbuf),
+				      gdk_pixbuf_get_width (mute_pixbuf),
+				      gdk_pixbuf_get_height (mute_pixbuf),
 				      0,
 				      0,
 				      1.0,
@@ -409,7 +405,7 @@ scale_key_press_event_cb (GtkWidget *widget, GdkEventKey *event, MixerData *data
 static gboolean
 scale_button_release_event_cb (GtkWidget *widget, GdkEventButton *event, MixerData *data)
 {
-	if (event->button == 1 && data->popup) {
+	if (data->popup != NULL) {
 		mixer_popup_hide (data, FALSE);
 		return TRUE;
 	}
@@ -418,19 +414,21 @@ scale_button_release_event_cb (GtkWidget *widget, GdkEventButton *event, MixerDa
 }
 
 static gboolean
-event_box_button_press_event_cb (GtkWidget *widget, GdkEventButton *event, MixerData *data)
+button_press_event_cb (GtkWidget *widget, GdkEventButton *event, MixerData *data)
 {
-	if (event->button == 1) {
-		if (!data->popup) {
-			mixer_popup_show (data);
-		}
-		else {
-			mixer_popup_hide (data, FALSE);
+	if (data->popup != NULL) {
+		mixer_popup_hide (data, FALSE);
+		return TRUE;
+	} else {
+		if (event->button == 1) {
+			if (event->type == GDK_2BUTTON_PRESS) {
+				mixer_start_gmix_cb (NULL, data, NULL);
+			} else {
+				mixer_popup_show (data);
+			}
 		}
 	}
 
-	g_signal_stop_emission_by_name (widget, "button_press_event");
-
 	if (event->button == 2) {
 		g_print ("Button2: Why won't the panel get this event?\n");
 	}
@@ -464,12 +462,13 @@ mixer_popup_show (MixerData *data)
 	
 	if (IS_PANEL_HORIZONTAL (data->orientation)) {
 		data->scale = gtk_vscale_new (data->adj);
-		gtk_widget_set_usize (data->scale, -1, 100);			
+		gtk_widget_set_size_request (data->scale, -1, 100);			
 	} else {
 		data->scale = gtk_hscale_new (data->adj);
-		gtk_widget_set_usize (data->scale, 100, -1);
+		gtk_widget_set_size_request (data->scale, 100, -1);
+		gtk_range_set_inverted (GTK_RANGE (data->scale), TRUE);
 	}
-	
+
 	g_signal_connect (data->scale,
 			  "button-release-event",
 			  (GCallback) scale_button_release_event_cb,
@@ -499,7 +498,7 @@ mixer_popup_show (MixerData *data)
 	gtk_widget_size_request (data->popup, &req);
 	
 	gdk_window_get_origin (data->image->window, &x, &y);
-	gdk_window_get_size (data->image->window, &width, &height);
+	gdk_drawable_get_size (data->image->window, &width, &height);
 	
 	switch (data->orientation) {
 	case PANEL_APPLET_ORIENT_DOWN:
@@ -616,30 +615,13 @@ destroy_mixer_cb (GtkWidget *widget, MixerData *data)
 		data->timeout = 0;
 	}
 
-	if (data->zero_pixbuf) {
-		g_object_unref (data->zero_pixbuf);
-		data->zero_pixbuf = NULL;
-	}
-	if (data->min_pixbuf) {
-		g_object_unref (data->min_pixbuf);
-		data->min_pixbuf = NULL;
-	}
-	if (data->medium_pixbuf) {
-		g_object_unref (data->medium_pixbuf);
-		data->medium_pixbuf = NULL;
-	}
-	if (data->max_pixbuf) {
-		g_object_unref (data->max_pixbuf);
-		data->max_pixbuf = NULL;
-	}
-	if (data->mute_pixbuf) {
-		g_object_unref (data->mute_pixbuf);
-		data->mute_pixbuf = NULL;
-	}
-	
 	g_free (data);
 
-	return;
+	num_applets--;
+	g_print ("num left %d\n", num_applets);
+	if (num_applets == 0) {
+		bonobo_main_quit ();
+	}
 }
 
 static void
@@ -652,10 +634,10 @@ applet_change_background_cb (PanelApplet               *applet,
 	if (type == PANEL_NO_BACKGROUND) {
 		GtkRcStyle *rc_style = gtk_rc_style_new ();
 
-		gtk_widget_modify_style (data->event_box, rc_style);
+		gtk_widget_modify_style (data->applet, rc_style);
 	}
 	else if (type == PANEL_COLOR_BACKGROUND) {
-		gtk_widget_modify_bg (data->event_box,
+		gtk_widget_modify_bg (data->applet,
 				      GTK_STATE_NORMAL,
 				      color);
 	} else { /* pixmap */
@@ -666,8 +648,6 @@ applet_change_background_cb (PanelApplet               *applet,
 static void
 applet_change_orient_cb (GtkWidget *w, PanelAppletOrient o, MixerData *data)
 {
-	gint vol;
-	
 	mixer_popup_hide (data, FALSE);
 
 	data->orientation = o;
@@ -684,11 +664,11 @@ applet_change_size_cb (GtkWidget *w, gint size, MixerData *data)
 	 * but we could scale up for the bigger ones...
 	 */
 	if (IS_PANEL_HORIZONTAL (data->orientation)) {
-		gtk_widget_set_size_request (GTK_WIDGET (data->event_box),
+		gtk_widget_set_size_request (GTK_WIDGET (data->frame),
 					     23,
 					     MIN (23, size));
 	} else {
-		gtk_widget_set_size_request (GTK_WIDGET (data->event_box),
+		gtk_widget_set_size_request (GTK_WIDGET (data->frame),
 					     MIN (23, size),
 					     23);
 	}
@@ -696,15 +676,15 @@ applet_change_size_cb (GtkWidget *w, gint size, MixerData *data)
 
 static void
 mixer_start_gmix_cb (BonoboUIComponent *uic,
-		     MixerData         *data,
+		     gpointer           data,
 		     const gchar       *verbname)
 {
-	gnome_execute_shell (NULL, "gmix");
+	gnome_execute_shell (NULL, "gnome-volume-control");
 }
 
 static void
 mixer_about_cb (BonoboUIComponent *uic,
-		MixerData         *data,
+		gpointer           data,
 		const gchar       *verbname)
 {
         static GtkWidget   *about     = NULL;
@@ -714,8 +694,7 @@ mixer_about_cb (BonoboUIComponent *uic,
         };
 
         if (about != NULL) {
-                gdk_window_show (about->window);
-                gdk_window_raise (about->window);
+                gtk_window_present (GTK_WINDOW (about));
                 return;
         }
 
@@ -727,16 +706,16 @@ mixer_about_cb (BonoboUIComponent *uic,
 				 NULL,
                                  NULL);
 
-        gtk_signal_connect (GTK_OBJECT (about), "destroy",
-                            GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &about);
+        g_signal_connect (G_OBJECT (about), "destroy",
+                          G_CALLBACK (gtk_widget_destroyed),
+	                  &about);
 	
         gtk_widget_show (about);
 }
 
 static void
 mixer_help_cb (BonoboUIComponent *uic,
-	       MixerData         *data,
+	       gpointer           data,
 	       const gchar       *verbname)
 {
 #if 0
@@ -752,7 +731,7 @@ mixer_help_cb (BonoboUIComponent *uic,
 /* Dummy callback to get rid of a warning, for now. */
 static void
 mixer_mute_cb (BonoboUIComponent *uic,
-	       MixerData         *data,
+	       gpointer           data,
 	       const gchar       *verbname)
 {
 }
@@ -790,32 +769,30 @@ mixer_ui_component_event (BonoboUIComponent            *comp,
 	}
 }
 
-static const BonoboUIVerb mixer_applet_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("RunMixer", mixer_start_gmix_cb),
-	BONOBO_UI_UNSAFE_VERB ("Mute",     mixer_mute_cb),
-	BONOBO_UI_UNSAFE_VERB ("Help",     mixer_help_cb),
-	BONOBO_UI_UNSAFE_VERB ("About",    mixer_about_cb),
-
+const BonoboUIVerb mixer_applet_menu_verbs [] = {
+	BONOBO_UI_VERB ("RunMixer", mixer_start_gmix_cb),
+	BONOBO_UI_VERB ("Mute",     mixer_mute_cb),
+	BONOBO_UI_VERB ("Help",     mixer_help_cb),
+	BONOBO_UI_VERB ("About",    mixer_about_cb),
+	
         BONOBO_UI_VERB_END
 };
 
-static const char mixer_applet_menu_xml [] =
+const gchar mixer_applet_menu_xml [] =
 "<popup name=\"button3\">\n"
-"   <menuitem name=\"RunMixer\" verb=\"RunMixer\" _label=\"Run Audio Mixer...\"\n"
-"             />" /*pixtype=\"stock\" pixname=\"gtk-volume\"/>\n"*/
-"   <menuitem name=\"Mute\" verb=\"Mute\" type=\"toggle\" _label=\"Mute\"\n"
-"             />" /*pixtype=\"stock\" pixname=\"gtk-volume\"/>\n"*/
+"   <menuitem name=\"RunMixer\" verb=\"RunMixer\" _label=\"Run Audio Mixer...\"/>"
+"   <menuitem name=\"Mute\" verb=\"Mute\" type=\"toggle\" _label=\"Mute\"/>"
 "   <menuitem name=\"Help\" verb=\"Help\" _label=\"Help\"\n"
 "             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
-"   <menuitem name=\"About\" verb=\"About\" _label=\"About ...\"\n"
+"   <menuitem name=\"About\" verb=\"About\" _label=\"About...\"\n"
 "             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
 "</popup>\n";
 
-static gboolean
-mixer_applet_fill (PanelApplet *applet)
+
+static void
+mixer_applet_create (PanelApplet *applet)
 {
 	MixerData         *data;
-	gint               vol;
 	BonoboUIComponent *component;
 
 #ifdef OSS_API
@@ -827,29 +804,27 @@ mixer_applet_fill (PanelApplet *applet)
 
 	data = g_new0 (MixerData, 1);
 
-	data->zero_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_zero_xpm);
-	data->min_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_min_xpm);
-	data->medium_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_medium_xpm);
-	data->max_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_max_xpm);
-	data->mute_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_mute_xpm);
-	
-	data->event_box = gtk_event_box_new ();
+	if (zero_pixbuf == NULL) {
+		zero_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_zero_xpm);
+		min_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_min_xpm);
+		medium_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_medium_xpm);
+		max_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_max_xpm);
+		mute_pixbuf = gdk_pixbuf_new_from_xpm_data (volume_mute_xpm);
+	}
 	
 	data->frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (data->frame), GTK_SHADOW_NONE);
-
-	gtk_container_add (GTK_CONTAINER (data->event_box), data->frame);
+	gtk_container_add (GTK_CONTAINER (applet), data->frame);
 	
 	data->image = gtk_image_new ();
-
-	g_signal_connect (data->event_box,
-			  "button-press-event",
-			  (GCallback) event_box_button_press_event_cb,
-			  data);
-	
 	gtk_container_add (GTK_CONTAINER (data->frame), data->image);
 
-	gtk_widget_show_all (data->event_box);
+	data->applet = GTK_WIDGET (applet);
+	
+	g_signal_connect (data->applet,
+			  "button-press-event",
+			  (GCallback) button_press_event_cb,
+			  data);
 	
         data->adj = GTK_ADJUSTMENT (
 		gtk_adjustment_new (-50,
@@ -859,19 +834,14 @@ mixer_applet_fill (PanelApplet *applet)
 				    VOLUME_MAX/10,
 				    0.0));
 
-	g_signal_connect (GTK_OBJECT (data->adj),
+	g_signal_connect (data->adj,
 			  "value-changed",
 			  (GCallback) mixer_value_changed_cb,
 			  data);
 
 	/* Get the initial volume. */
-	vol = readMixer ();
-	data->vol = vol;
+	data->vol = readMixer ();
 
-	gtk_container_add (GTK_CONTAINER (applet), data->event_box);
-	
-	data->applet = GTK_WIDGET (applet);
-	
 	/* Install timeout handler, that keeps the applet up-to-date if the
 	 * volume is changed somewhere else.
 	 */
@@ -881,51 +851,48 @@ mixer_applet_fill (PanelApplet *applet)
 
 	data->mute = FALSE;
 
-	g_signal_connect (G_OBJECT (applet),
+	g_signal_connect (applet,
 			  "destroy",
 			  (GCallback) destroy_mixer_cb,
 			  data);
 
-	g_signal_connect (G_OBJECT (applet),
+	g_signal_connect (applet,
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (applet),
+	g_signal_connect (applet,
 			  "change_size",
 			  G_CALLBACK (applet_change_size_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (applet),
+	g_signal_connect (applet,
 			  "change_background",
 			  G_CALLBACK (applet_change_background_cb),
 			  data);
-	
-	panel_applet_setup_menu (PANEL_APPLET (applet),
+
+	panel_applet_setup_menu (PANEL_APPLET (data->applet),
 				 mixer_applet_menu_xml,
 				 mixer_applet_menu_verbs,
 				 data);
 
-	component = panel_applet_get_popup_component (PANEL_APPLET (applet));
-
+	component = panel_applet_get_popup_component (PANEL_APPLET (data->applet));
 	g_signal_connect (component,
 			  "ui-event",
 			  (GCallback) mixer_ui_component_event,
 			  data);
-	
+
 	applet_change_orient_cb (GTK_WIDGET (applet),
-				 panel_applet_get_orient (PANEL_APPLET (applet)),
+				 panel_applet_get_orient (applet),
 				 data);
 	applet_change_size_cb (GTK_WIDGET (applet),
-			       panel_applet_get_size (PANEL_APPLET (applet)),
+			       panel_applet_get_size (applet),
 			       data);
-	
+
 	mixer_update_slider (data);
 	mixer_update_image (data);
 
-	gtk_widget_show (GTK_WIDGET (applet));
-
-	return TRUE;
+	gtk_widget_show_all (GTK_WIDGET (applet));
 }
 
 static gboolean
@@ -933,15 +900,26 @@ mixer_applet_factory (PanelApplet *applet,
 		      const gchar *iid,
 		      gpointer     data)
 {
-	gboolean retval = FALSE;
-	
-	if (!strcmp (iid, "OAFIID:GNOME_MixerApplet"))
-		retval = mixer_applet_fill (applet);
-	
-	return retval;
+#ifdef ENABLE_NLS
+	static inited = FALSE;
+
+	if (!inited) {
+		bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+		textdomain(GETTEXT_PACKAGE);
+
+		inited = TRUE;
+	}
+#endif	
+
+	mixer_applet_create (applet);
+	num_applets++;
+
+	return TRUE;
 }
 
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_MixerApplet_Factory",
+			     PANEL_TYPE_APPLET,
 			     "Volume control applet",
 			     "0",
 			     mixer_applet_factory,
