@@ -20,54 +20,18 @@
 
 /* If you think this code is a mess, you should see my flat ;) */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <panel-applet.h>
 #include "screenshooter_applet.h"
-#include <libgnomeui/gnome-window-icon.h>
 
+static void grab_shot (user_preferences * opt, gboolean root);
+
+#ifdef FIXME
 void showHelp (AppletWidget * applet, gpointer data);
 
-void
-cb_about (AppletWidget * widget, gpointer data)
-{
-  static GtkWidget *about = NULL;
-  GtkWidget *my_url;
-  static const char *authors[] =
-    { "Tom Gilbert <gilbertt@tomgilbert.freeserve.co.uk>",
-    "Telsa Gwynne <hobbit@aloss.ukuu.org.uk> (typo fixes, documentation)",
-    "The Sirius Cybernetics Corporation <Ursa-minor>", NULL
-  };
-
-  if (about != NULL)
-  {
-  	gdk_window_show(about->window);
-	gdk_window_raise(about->window);
-	return;	
-  }
-
-  about = gnome_about_new
-    (_ ("Screen-Shooter Applet"),
-     VERSION,
-     _ ("Copyright (C) 1999 Tom Gilbert"),
-     authors, _ ("A useful little screengrabber.\n"
-		 "The left/top button allows you to grab either:\n"
-		 "a window (click to select which one)\n"
-		 "or an area (click and drag to select a rectangle.)\n"
-		 "The right/bottom button will grab the entire desktop.\n"
-		 "Share and Enjoy ;)"), NULL);
-  gtk_signal_connect( GTK_OBJECT(about), "destroy",
-		      GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about );
-
-  my_url = gnome_href_new ("http://www.linuxbrit.co.uk/",
-			   _("Visit the author's Website"));
-  gtk_widget_show (my_url);
-  gtk_box_pack_start (GTK_BOX ((GNOME_DIALOG (about))->vbox), my_url, FALSE,
-		      FALSE, 0);
-
-  gtk_widget_show (about);
-  data = NULL;
-  widget = NULL;
-}
-
 static void
 properties_save (gchar * path, gpointer data)
 {
@@ -100,31 +64,31 @@ properties_save (gchar * path, gpointer data)
   gnome_config_set_string ("screenshooter/thumb_filename",
 			   options.thumb_filename);
   gnome_config_set_string ("screenshooter/app", options.app);
-  gnome_config_set_bool ("screenshoter/decoration", options.decoration);
-  gnome_config_set_bool ("screenshoter/monochrome", options.monochrome);
-  gnome_config_set_bool ("screenshoter/negate", options.negate);
-  gnome_config_set_bool ("screenshoter/view", options.view);
-  gnome_config_set_bool ("screenshoter/equalize", options.equalize);
-  gnome_config_set_bool ("screenshoter/normalize", options.normalize);
-  gnome_config_set_bool ("screenshoter/beep", options.beep);
-  gnome_config_set_bool ("screenshoter/frame", options.frame);
-  gnome_config_set_bool ("screenshoter/thumb", options.thumb);
-  gnome_config_set_bool ("screenshoter/flip", options.flip);
-  gnome_config_set_bool ("screenshoter/flop", options.flop);
-  gnome_config_set_bool ("screenshoter/despeckle", options.despeckle);
-  gnome_config_set_bool ("screenshoter/enhance", options.enhance);
-  gnome_config_set_bool ("screenshoter/use_script", options.use_script);
-  gnome_config_set_bool ("screenshoter/emboss", options.emboss);
-  gnome_config_set_bool ("screenshoter/blur", options.blur);
-  gnome_config_set_bool ("screenshoter/charcoal", options.charcoal);
-  gnome_config_set_bool ("screenshoter/edge", options.edge);
-  gnome_config_set_bool ("screenshoter/implode", options.implode);
-  gnome_config_set_bool ("screenshoter/paint", options.paint);
-  gnome_config_set_bool ("screenshoter/spurious", options.spurious);
-  gnome_config_set_bool ("screenshoter/solarize", options.solarize);
-  gnome_config_set_bool ("screenshoter/spread", options.spread);
-  gnome_config_set_bool ("screenshoter/swirl", options.swirl);
-  gnome_config_set_bool ("screenshoter/thumbnail_intermediate",
+  gnome_config_set_bool ("screenshooter/decoration", options.decoration);
+  gnome_config_set_bool ("screenshooter/monochrome", options.monochrome);
+  gnome_config_set_bool ("screenshooter/negate", options.negate);
+  gnome_config_set_bool ("screenshooter/view", options.view);
+  gnome_config_set_bool ("screenshooter/equalize", options.equalize);
+  gnome_config_set_bool ("screenshooter/normalize", options.normalize);
+  gnome_config_set_bool ("screenshooter/beep", options.beep);
+  gnome_config_set_bool ("screenshooter/frame", options.frame);
+  gnome_config_set_bool ("screenshooter/thumb", options.thumb);
+  gnome_config_set_bool ("screenshooter/flip", options.flip);
+  gnome_config_set_bool ("screenshooter/flop", options.flop);
+  gnome_config_set_bool ("screenshooter/despeckle", options.despeckle);
+  gnome_config_set_bool ("screenshooter/enhance", options.enhance);
+  gnome_config_set_bool ("screenshooter/use_script", options.use_script);
+  gnome_config_set_bool ("screenshooter/emboss", options.emboss);
+  gnome_config_set_bool ("screenshooter/blur", options.blur);
+  gnome_config_set_bool ("screenshooter/charcoal", options.charcoal);
+  gnome_config_set_bool ("screenshooter/edge", options.edge);
+  gnome_config_set_bool ("screenshooter/implode", options.implode);
+  gnome_config_set_bool ("screenshooter/paint", options.paint);
+  gnome_config_set_bool ("screenshooter/spurious", options.spurious);
+  gnome_config_set_bool ("screenshooter/solarize", options.solarize);
+  gnome_config_set_bool ("screenshooter/spread", options.spread);
+  gnome_config_set_bool ("screenshooter/swirl", options.swirl);
+  gnome_config_set_bool ("screenshooter/thumbnail_intermediate",
 			 options.thumbnail_intermediate);
   gnome_config_pop_prefix ();
   gnome_config_sync ();
@@ -133,9 +97,12 @@ properties_save (gchar * path, gpointer data)
   data = NULL;
 }
 
+#endif
+
 static void
 properties_load (gchar * path, gpointer data)
 {
+#ifdef FIXME
   gnome_config_push_prefix (path);
   options.propwindow = NULL;
   options.quality_entry = NULL;
@@ -218,6 +185,65 @@ properties_load (gchar * path, gpointer data)
   options.thumbnail_intermediate =
     gnome_config_get_bool ("screenshooter/thumbnail_intermediate=FALSE");
   gnome_config_pop_prefix ();
+#endif  
+  options.propwindow = NULL;
+  options.quality_entry = NULL;
+  options.directory_entry = NULL;
+  options.filename_entry = NULL;
+  options.delay_entry = NULL;
+  options.app_entry = NULL;
+  options.script_entry = NULL;
+  options.thumb_filename_entry = NULL;
+  options.spurious_pref_vbox = NULL;
+  options.spurious_pref_vbox2 = NULL;
+  options.spurious_pref_vbox3 = NULL;
+
+  options.quality = 75;
+  options.rotate_degrees =0;
+  options.blur_factor = 0;
+  options.charcoal_factor =0;
+  options.edge_factor = 0;
+  options.implode_factor =0;
+  options.paint_radius =0;
+  options.sharpen_factor =0;
+  options.solarize_factor =0;
+  options.spread_radius =0;
+  options.swirl_degrees =0;
+  options.frame_size = 6;
+  options.thumb_quality = 50;
+  options.thumb_size = 25;
+  options.gamma_factor = 1.6;
+  options.delay = 0;
+  options.directory = "~/";
+  options.filename ="`date +%Y_%m_%d_%H%M%S`_shot.jpg";
+  options.thumb_filename = g_strdup ("thumb-");
+  options.script_filename = g_strdup ("update_scrshot_page");
+  options.app =  g_strdup ("display");
+  options.decoration =TRUE;
+  options.monochrome = FALSE;
+  options.negate = FALSE;
+  options.view = FALSE;
+  options.beep = TRUE;
+  options.thumb = FALSE;
+  options.frame = FALSE;
+  options.equalize = FALSE;
+  options.normalize = FALSE;
+  options.flip = FALSE;
+  options.flop = FALSE;
+  options.enhance = FALSE;
+  options.emboss = FALSE;
+  options.blur =  FALSE;
+  options.spurious = FALSE;
+  options.charcoal = FALSE;
+  options.edge =  FALSE;
+  options.implode = FALSE;
+  options.paint = FALSE;
+  options.solarize = FALSE;
+  options.spread = FALSE;
+  options.swirl =  FALSE;
+  options.despeckle = FALSE;
+  options.use_script =FALSE;
+  options.thumbnail_intermediate =FALSE;
   return;
   data = NULL;
 }
@@ -229,13 +255,15 @@ set_tooltip (GtkWidget * w, const gchar * tip)
   gtk_tooltips_set_tip (t, w, tip, NULL);
 }
 
-gboolean need_to_change_orientation (PanelOrientType o, gboolean size_tiny)
+
+static gboolean 
+need_to_change_orientation (PanelAppletOrient o, gboolean size_tiny)
 {
   gboolean need_to = FALSE;
   switch (o)
     {
-    case ORIENT_UP:
-    case ORIENT_DOWN:
+    case PANEL_APPLET_ORIENT_UP:
+    case PANEL_APPLET_ORIENT_DOWN:
       {
 	if ((size_tiny)
 	    && ((GTK_WIDGET_VISIBLE (vframe))
@@ -247,8 +275,8 @@ gboolean need_to_change_orientation (PanelOrientType o, gboolean size_tiny)
 	  need_to = TRUE;
 	break;
       }
-    case ORIENT_LEFT:
-    case ORIENT_RIGHT:
+    case PANEL_APPLET_ORIENT_LEFT:
+    case PANEL_APPLET_ORIENT_RIGHT:
       {
 	if ((size_tiny)
 	    && ((GTK_WIDGET_VISIBLE (hframe))
@@ -269,16 +297,16 @@ gboolean need_to_change_orientation (PanelOrientType o, gboolean size_tiny)
   return need_to;
 }
 
-void
-change_orientation (PanelOrientType o, gboolean panel_size_tiny)
+static void
+change_orientation (PanelAppletOrient o, gboolean panel_size_tiny)
 {
   if (need_to_change_orientation (o, panel_size_tiny))
     {
       gtk_widget_hide (applet);
       switch (o)
 	{
-	case ORIENT_UP:
-	case ORIENT_DOWN:
+	case PANEL_APPLET_ORIENT_UP:
+	case PANEL_APPLET_ORIENT_DOWN:
 	  {
 	    if (panel_size_tiny)
 	      {
@@ -292,8 +320,8 @@ change_orientation (PanelOrientType o, gboolean panel_size_tiny)
 	      }
 	    break;
 	  }
-	case ORIENT_LEFT:
-	case ORIENT_RIGHT:
+	case PANEL_APPLET_ORIENT_LEFT:
+	case PANEL_APPLET_ORIENT_RIGHT:
 	  {
 	    if (panel_size_tiny)
 	      {
@@ -317,52 +345,51 @@ change_orientation (PanelOrientType o, gboolean panel_size_tiny)
     }
 }
 
-void
-cb_applet_change_orient (GtkWidget * w, PanelOrientType o, gpointer data)
+static void
+cb_applet_change_orient (PanelApplet *applet, PanelAppletOrient  o, gpointer data)
 {
   gboolean size_tiny = FALSE;
-#ifdef HAVE_PANEL_PIXEL_SIZE
-  int panelsize = applet_widget_get_panel_pixel_size (APPLET_WIDGET (applet));
+ 
+  int panelsize = panel_applet_get_size (applet);
   /* Switch stuff so it lies flat on a tiny panel */
-  if (panelsize < PIXEL_SIZE_STANDARD)
+  if (panelsize < GNOME_Vertigo_PANEL_MEDIUM)
     size_tiny = TRUE;
   else
     size_tiny = FALSE;
-#endif
+
 
   change_orientation (o, size_tiny);
 
-  w = NULL;
   data = NULL;
 }
 
-#ifdef HAVE_PANEL_PIXEL_SIZE
+
 static void
-applet_change_pixel_size (GtkWidget * w, int s, gpointer data)
+applet_change_pixel_size (PanelApplet *applet, gint s, gpointer data)
 {
   gboolean size_tiny = FALSE;
-  PanelOrientType orient =
-    applet_widget_get_panel_orient (APPLET_WIDGET (applet));
+  PanelAppletOrient orient =
+    panel_applet_get_orient (applet);
 
   /* Switch stuff so it lies flat on a tiny panel */
-  if (s < PIXEL_SIZE_STANDARD)
+  if (s < GNOME_Vertigo_PANEL_MEDIUM)
     size_tiny = TRUE;
   else
     size_tiny = FALSE;
 
   change_orientation (orient, size_tiny);
 
-  w = NULL;
   data = NULL;
 }
-#endif
 
-void
-cb_properties_dialog (AppletWidget * widget, gpointer data)
+static void
+cb_properties_dialog (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
 
   user_preferences *ad = &options;
+#ifdef FIXME
   static GnomeHelpMenuEntry helpEntry = { NULL, "preferences" };
+#endif
   GtkWidget *frame;
   GtkWidget *pref_vbox;
   GtkWidget *pref_vbox_2;
@@ -377,9 +404,9 @@ cb_properties_dialog (AppletWidget * widget, gpointer data)
       gdk_window_raise (ad->propwindow->window);
       return;
     }
-
+#ifdef FIXME
   helpEntry.name = gnome_app_id;
-
+#endif
   /* Grab old property settings for comparison */
   memcpy (&old_options, &options, sizeof (user_preferences));
 
@@ -450,8 +477,9 @@ cb_properties_dialog (AppletWidget * widget, gpointer data)
 
   ad->directory_entry = gnome_file_entry_new ("output_dir",
 					      "Select directory");
-  gnome_file_entry_set_directory (GNOME_FILE_ENTRY (ad->directory_entry),
-				  TRUE);
+  gnome_file_entry_set_directory_entry (GNOME_FILE_ENTRY (ad->directory_entry),
+				        TRUE);
+#ifdef FIXME  
   entry = GTK_COMBO (GNOME_FILE_ENTRY (ad->directory_entry)->gentry)->entry;
 
   if (ad->directory)
@@ -460,7 +488,7 @@ cb_properties_dialog (AppletWidget * widget, gpointer data)
   gtk_signal_connect_object (GTK_OBJECT (entry), "changed",
 			     GTK_SIGNAL_FUNC (gnome_property_box_changed),
 			     GTK_OBJECT (ad->propwindow));
-
+#endif
   gtk_box_pack_start (GTK_BOX (pref_hbox), ad->directory_entry, TRUE, TRUE,
 		      0);
   gtk_widget_show (ad->directory_entry);
@@ -851,9 +879,10 @@ cb_properties_dialog (AppletWidget * widget, gpointer data)
 		      GTK_SIGNAL_FUNC (property_apply_cb), ad);
   gtk_signal_connect (GTK_OBJECT (ad->propwindow), "destroy",
 		      GTK_SIGNAL_FUNC (property_destroy_cb), ad);
+#ifdef FIXME
   gtk_signal_connect (GTK_OBJECT (ad->propwindow), "help",
 		      GTK_SIGNAL_FUNC (gnome_help_pbox_display), &helpEntry);
-
+#endif
   if (ad->spurious)
     {
       gtk_widget_show_all (ad->spurious_pref_vbox);
@@ -863,9 +892,9 @@ cb_properties_dialog (AppletWidget * widget, gpointer data)
 
   gtk_widget_show (ad->propwindow);
   data = NULL;
-  widget = NULL;
 }
 
+#ifdef FIXME
 static gint
 applet_save_session (GtkWidget * widget, gchar * privcfgpath,
 		     gchar * globcfgpath, gpointer data)
@@ -876,164 +905,9 @@ applet_save_session (GtkWidget * widget, gchar * privcfgpath,
   globcfgpath = NULL;
   data = NULL;
 }
-
-int
-main (int argc, char *argv[])
-{
-  GtkWidget *hbox;
-  GtkWidget *vbox;
-  GtkWidget *mainbox;
-  GtkWidget *hwindow_button, *hroot_button;
-  GtkWidget *vwindow_button, *vroot_button;
-  GtkWidget *hwindowpixmap;
-  GtkWidget *hrootpixmap;
-  GtkWidget *vwindowpixmap;
-  GtkWidget *vrootpixmap;
-  PanelOrientType orient;
-  gboolean init_size_tiny = FALSE;
-#ifdef HAVE_PANEL_PIXEL_SIZE
-  int init_size;
 #endif
 
-  /* Initialize the i18n stuff */
-  bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-  textdomain (PACKAGE);
-
-  applet_widget_init ("screenshooter_applet", VERSION, argc, argv, NULL, 0,
-		      NULL);
-  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/screenshooter_applet.png");
-
-  applet = applet_widget_new ("screenshooter_applet");
-  if (!applet)
-    g_error (_ ("Can't create applet!\n"));
-
-  gtk_widget_realize (applet);
-
-  properties_load (APPLET_WIDGET (applet)->privcfgpath, NULL);
-
-  mainbox = gtk_vbox_new (FALSE, 0);
-  hbox = gtk_hbox_new (FALSE, 1);
-  vbox = gtk_vbox_new (FALSE, 1);
-
-  vframe = gtk_frame_new (NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (vframe), 0);
-  gtk_frame_set_shadow_type (GTK_FRAME (vframe), GTK_SHADOW_IN);
-
-  hframe = gtk_frame_new (NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (hframe), 0);
-  gtk_frame_set_shadow_type (GTK_FRAME (hframe), GTK_SHADOW_IN);
-
-  gtk_container_add (GTK_CONTAINER (hframe), hbox);
-  gtk_container_add (GTK_CONTAINER (vframe), vbox);
-  gtk_box_pack_start (GTK_BOX (mainbox), hframe, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (mainbox), vframe, FALSE, FALSE, 0);
-
-  hwindow_button = gtk_button_new ();
-  vwindow_button = gtk_button_new ();
-  hroot_button = gtk_button_new ();
-  vroot_button = gtk_button_new ();
-  gtk_widget_set_usize (hwindow_button, 18, 18);
-  gtk_widget_set_usize (vwindow_button, 18, 18);
-  gtk_widget_set_usize (hroot_button, 18, 18);
-  gtk_widget_set_usize (vroot_button, 18, 18);
-  gtk_box_pack_start (GTK_BOX (hbox), hwindow_button, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), vwindow_button, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), hroot_button, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), vroot_button, TRUE, TRUE, 0);
-  gtk_signal_connect (GTK_OBJECT
-		      (hwindow_button), "clicked",
-		      GTK_SIGNAL_FUNC (window_button_press), &options);
-  gtk_signal_connect (GTK_OBJECT
-		      (hroot_button), "clicked",
-		      GTK_SIGNAL_FUNC (desktop_button_press), &options);
-  gtk_signal_connect (GTK_OBJECT
-		      (vwindow_button), "clicked",
-		      GTK_SIGNAL_FUNC (window_button_press), &options);
-  gtk_signal_connect (GTK_OBJECT
-		      (vroot_button), "clicked",
-		      GTK_SIGNAL_FUNC (desktop_button_press), &options);
-  GTK_WIDGET_UNSET_FLAGS (hwindow_button, GTK_CAN_DEFAULT);
-  GTK_WIDGET_UNSET_FLAGS (hroot_button, GTK_CAN_DEFAULT);
-  GTK_WIDGET_UNSET_FLAGS (hwindow_button, GTK_CAN_FOCUS);
-  GTK_WIDGET_UNSET_FLAGS (hroot_button, GTK_CAN_FOCUS);
-  GTK_WIDGET_UNSET_FLAGS (vwindow_button, GTK_CAN_DEFAULT);
-  GTK_WIDGET_UNSET_FLAGS (vroot_button, GTK_CAN_DEFAULT);
-  GTK_WIDGET_UNSET_FLAGS (vwindow_button, GTK_CAN_FOCUS);
-  GTK_WIDGET_UNSET_FLAGS (vroot_button, GTK_CAN_FOCUS);
-
-  set_tooltip (hwindow_button,
-	       (_ ("Grab a shot of a specific window or area")));
-  set_tooltip (hroot_button, (_ ("Grab a shot of your entire desktop")));
-  set_tooltip (vwindow_button,
-	       (_ ("Grab a shot of a specific window or area")));
-  set_tooltip (vroot_button, (_ ("Grab a shot of your entire desktop")));
-
-  vwindowpixmap = gnome_pixmap_new_from_xpm_d (window_icon_xpm);
-  vrootpixmap = gnome_pixmap_new_from_xpm_d (root_icon_xpm);
-  gtk_container_add (GTK_CONTAINER (vwindow_button), vwindowpixmap);
-  gtk_container_add (GTK_CONTAINER (vroot_button), vrootpixmap);
-
-  hwindowpixmap = gnome_pixmap_new_from_xpm_d (window_icon_xpm);
-  hrootpixmap = gnome_pixmap_new_from_xpm_d (root_icon_xpm);
-  gtk_container_add (GTK_CONTAINER (hwindow_button), hwindowpixmap);
-  gtk_container_add (GTK_CONTAINER (hroot_button), hrootpixmap);
-
-  gtk_widget_show (hwindowpixmap);
-  gtk_widget_show (vwindow_button);
-  gtk_widget_show (vroot_button);
-  gtk_widget_show (hwindow_button);
-  gtk_widget_show (hroot_button);
-  gtk_widget_show (hrootpixmap);
-  gtk_widget_show (vwindowpixmap);
-  gtk_widget_show (vrootpixmap);
-  gtk_widget_show (vbox);
-  gtk_widget_show (hbox);
-  gtk_widget_show (mainbox);
-
-  applet_widget_add (APPLET_WIDGET (applet), mainbox);
-
-  orient = applet_widget_get_panel_orient (APPLET_WIDGET (applet));
-#ifdef HAVE_PANEL_PIXEL_SIZE
-  init_size = applet_widget_get_panel_pixel_size (APPLET_WIDGET (applet));
-  if (init_size < PIXEL_SIZE_STANDARD)
-    init_size_tiny = TRUE;
-#endif
-  change_orientation (orient, init_size_tiny);
-
-  applet_widget_register_stock_callback (APPLET_WIDGET (applet),
-					 "properties",
-					 GNOME_STOCK_MENU_PROP,
-					 _ ("Properties..."),
-					 (AppletCallbackFunc)
-					 cb_properties_dialog, NULL);
-  applet_widget_register_stock_callback (APPLET_WIDGET (applet),
-					 "help",
-					 GNOME_STOCK_PIXMAP_HELP,
-					 _ ("Help"),
-					 (AppletCallbackFunc) showHelp, NULL);
-  applet_widget_register_stock_callback (APPLET_WIDGET (applet),
-					 "about",
-					 GNOME_STOCK_MENU_ABOUT,
-					 _ ("About..."),
-					 (AppletCallbackFunc) cb_about, NULL);
-
-  gtk_signal_connect (GTK_OBJECT (applet), "change_orient",
-		      GTK_SIGNAL_FUNC (cb_applet_change_orient), NULL);
-
-#ifdef HAVE_PANEL_PIXEL_SIZE
-  gtk_signal_connect (GTK_OBJECT (applet), "change_pixel_size",
-		      GTK_SIGNAL_FUNC (applet_change_pixel_size), NULL);
-#endif
-
-  gtk_signal_connect (GTK_OBJECT (applet), "save_session",
-		      GTK_SIGNAL_FUNC (applet_save_session), NULL);
-
-  gtk_widget_show (applet);
-  applet_widget_gtk_main ();
-  return 0;
-}
-
-void
+static void
 window_button_press (GtkWidget * button, user_preferences * opt)
 {
   grab_shot (opt, FALSE);
@@ -1041,7 +915,7 @@ window_button_press (GtkWidget * button, user_preferences * opt)
   button = NULL;
 }
 
-void
+static void
 desktop_button_press (GtkWidget * button, user_preferences * opt)
 {
   grab_shot (opt, TRUE);
@@ -1049,7 +923,8 @@ desktop_button_press (GtkWidget * button, user_preferences * opt)
   button = NULL;
 }
 
-void
+
+static void
 grab_shot (user_preferences * opt, gboolean root)
 {
   gchar *sys = NULL;
@@ -1438,9 +1313,7 @@ grab_shot (user_preferences * opt, gboolean root)
       gchar *cmd;
       if (opt->thumb)
 	{
-	  cmd =
-	    g_strdup_printf ("%s \"%s\" \"%s\" &", opt->app, lastimage,
-			     thumb_image);
+	  cmd = g_strdup_printf ("%s \"%s\" \"%s\" &", opt->app, lastimage, thumb_image);
 	}
       else
 	{
@@ -1488,13 +1361,15 @@ expand_cb (GtkWidget * w, gpointer data)
   gchar *message = NULL;
   gchar *buf = NULL;
 
-  filename = gtk_entry_get_text (GTK_ENTRY (options.filename_entry));
+  filename = gtk_editable_get_chars (GTK_EDITABLE (options.filename_entry), 0, -1);
 
   if ((wordexpret = wordexp (filename, &mywordexp, 0) != 0))
     {
-      gnome_ok_dialog (_ ("There was a word expansion error\n"
+      gnome_ok_dialog (_("There was a word expansion error\n"
 			  "I think you have stuck something funny in the filename box"));
       wordfree (&mywordexp);
+      if (filename)
+          g_free (filename);
       return;
     }
   else
@@ -1509,11 +1384,14 @@ expand_cb (GtkWidget * w, gpointer data)
       gnome_ok_dialog (message);
       wordfree (&mywordexp);
     }
+  if (filename)
+    g_free (filename);
   return;
   data = NULL;
   w = NULL;
 }
 
+
 void
 property_apply_cb (GtkWidget * w, gpointer data)
 {
@@ -1522,6 +1400,7 @@ property_apply_cb (GtkWidget * w, gpointer data)
   gchar *buf;
   gint info_changed = FALSE;
 
+#ifdef FIXME
   buf =
     gtk_entry_get_text (GTK_ENTRY
 			(GTK_COMBO
@@ -1536,8 +1415,8 @@ property_apply_cb (GtkWidget * w, gpointer data)
       ad->directory = g_strdup (buf);
       info_changed = TRUE;
     }
-
-  buf = gtk_entry_get_text (GTK_ENTRY (ad->filename_entry));
+#endif
+  buf = gtk_editable_get_chars (GTK_EDITABLE (ad->filename_entry), 0, -1);
 
   if ((buf) && (strcmp (buf, old->filename)))
     {
@@ -1547,8 +1426,10 @@ property_apply_cb (GtkWidget * w, gpointer data)
       ad->filename = g_strdup (buf);
       info_changed = TRUE;
     }
+  if (buf)
+    g_free (buf);
 
-  buf = gtk_entry_get_text (GTK_ENTRY (ad->app_entry));
+  buf = gtk_editable_get_chars (GTK_EDITABLE (ad->app_entry), 0, -1);
 
   if ((buf) && (strcmp (buf, old->app)))
     {
@@ -1558,8 +1439,10 @@ property_apply_cb (GtkWidget * w, gpointer data)
       ad->app = g_strdup (buf);
       info_changed = TRUE;
     }
+  if (buf)
+    g_free (buf);
 
-  buf = gtk_entry_get_text (GTK_ENTRY (ad->thumb_filename_entry));
+  buf = gtk_editable_get_chars (GTK_EDITABLE (ad->thumb_filename_entry), 0, -1);
 
   if ((buf) && (strcmp (buf, old->thumb_filename)))
     {
@@ -1569,8 +1452,10 @@ property_apply_cb (GtkWidget * w, gpointer data)
       ad->thumb_filename = g_strdup (buf);
       info_changed = TRUE;
     }
+  if (buf)
+    g_free (buf);
 
-  buf = gtk_entry_get_text (GTK_ENTRY (ad->script_entry));
+  buf =  gtk_editable_get_chars (GTK_EDITABLE (ad->script_entry), 0, -1);
 
   if ((buf) && (strcmp (buf, old->script_filename)))
     {
@@ -1580,7 +1465,9 @@ property_apply_cb (GtkWidget * w, gpointer data)
       ad->script_filename = g_strdup (buf);
       info_changed = TRUE;
     }
-
+  if (buf)
+    g_free (buf);
+   
   if (memcmp (ad, old, sizeof (user_preferences)))
     info_changed = TRUE;
 
@@ -1686,8 +1573,9 @@ create_slider_option (gchar * label, GtkWidget * target, int *option,
 }
 
 void
-showHelp (AppletWidget * applet, gpointer data)
+cb_help (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 {
+#ifdef FIXME
   static GnomeHelpMenuEntry help_entry = { NULL, "index.html" };
 
   help_entry.name = gnome_app_id;
@@ -1696,4 +1584,208 @@ showHelp (AppletWidget * applet, gpointer data)
   return;
   applet = NULL;
   data = NULL;
+#endif
 }
+
+static void
+cb_about (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+{
+  static GtkWidget *about = NULL;
+  GtkWidget *my_url;
+  static const char *authors[] =
+    { "Tom Gilbert <gilbertt@tomgilbert.freeserve.co.uk>",
+    "Telsa Gwynne <hobbit@aloss.ukuu.org.uk> (typo fixes, documentation)",
+    "The Sirius Cybernetics Corporation <Ursa-minor>", NULL
+  };
+
+  if (about != NULL)
+  {
+  	gdk_window_show(about->window);
+	gdk_window_raise(about->window);
+	return;	
+  }
+
+  about = gnome_about_new
+    (_ ("Screen-Shooter Applet"),
+     VERSION,
+     _ ("Copyright (C) 1999 Tom Gilbert"),
+     _ ("A useful little screengrabber.\n"
+	"The left/top button allows you to grab either:\n"
+	"a window (click to select which one)\n"
+	"or an area (click and drag to select a rectangle.)\n"
+	"The right/bottom button will grab the entire desktop.\n"
+	 "Share and Enjoy ;)"), 
+     authors, 
+     NULL,
+     NULL,
+     NULL);
+
+#ifdef FIXME  
+  my_url = gnome_href_new ("http://www.linuxbrit.co.uk/",
+			   _("Visit the author's Website"));
+  gtk_widget_show (my_url);
+  gtk_box_pack_start (GTK_BOX ((GNOME_DIALOG (about))->vbox), my_url, FALSE,
+		      FALSE, 0);
+#endif
+
+  gtk_widget_show (about);
+  
+}
+
+static const BonoboUIVerb test_applet_menu_verbs [] = {
+        BONOBO_UI_VERB ("Props", cb_properties_dialog),
+        BONOBO_UI_VERB ("Help", cb_help),
+        BONOBO_UI_VERB ("About", cb_about),
+
+        BONOBO_UI_VERB_END
+};
+
+static const char test_applet_menu_xml [] =
+	"<popup name=\"button3\">\n"
+	"   <menuitem name=\"Item 1\" verb=\"Props\" _label=\"Properties\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
+	"   <menuitem name=\"Item 2\" verb=\"Help\" _label=\"Help\"\n"
+	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
+	"   <menuitem name=\"Item 3\" verb=\"About\" _label=\"About\"\n"
+	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"	
+	"</popup>\n";
+
+
+static gboolean 
+screenshooter_applet_fill (PanelApplet *applet)
+{
+  GtkWidget *hbox;
+  GtkWidget *vbox;
+  GtkWidget *mainbox;
+  GtkWidget *hwindow_button, *hroot_button;
+  GtkWidget *vwindow_button, *vroot_button;
+  GtkWidget *hwindowpixmap;
+  GtkWidget *hrootpixmap;
+  GtkWidget *vwindowpixmap;
+  GtkWidget *vrootpixmap;
+  
+	
+  properties_load (NULL, NULL);
+
+  mainbox = gtk_vbox_new (FALSE, 0);
+  hbox = gtk_hbox_new (FALSE, 1);
+  vbox = gtk_vbox_new (FALSE, 1);
+
+  vframe = gtk_frame_new (NULL);
+  gtk_container_set_border_width (GTK_CONTAINER (vframe), 0);
+  gtk_frame_set_shadow_type (GTK_FRAME (vframe), GTK_SHADOW_IN);
+
+  hframe = gtk_frame_new (NULL);
+  gtk_container_set_border_width (GTK_CONTAINER (hframe), 0);
+  gtk_frame_set_shadow_type (GTK_FRAME (hframe), GTK_SHADOW_IN);
+
+  gtk_container_add (GTK_CONTAINER (hframe), hbox);
+  gtk_container_add (GTK_CONTAINER (vframe), vbox);
+  gtk_box_pack_start (GTK_BOX (mainbox), hframe, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (mainbox), vframe, FALSE, FALSE, 0);
+
+  hwindow_button = gtk_button_new ();
+  vwindow_button = gtk_button_new ();
+  hroot_button = gtk_button_new ();
+  vroot_button = gtk_button_new ();
+  gtk_widget_set_usize (hwindow_button, 18, 18);
+  gtk_widget_set_usize (vwindow_button, 18, 18);
+  gtk_widget_set_usize (hroot_button, 18, 18);
+  gtk_widget_set_usize (vroot_button, 18, 18);
+  gtk_box_pack_start (GTK_BOX (hbox), hwindow_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), vwindow_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), hroot_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), vroot_button, TRUE, TRUE, 0);
+
+  gtk_signal_connect (GTK_OBJECT
+		      (hwindow_button), "clicked",
+		      GTK_SIGNAL_FUNC (window_button_press), &options);
+  gtk_signal_connect (GTK_OBJECT
+		      (hroot_button), "clicked",
+		      GTK_SIGNAL_FUNC (desktop_button_press), &options);
+  gtk_signal_connect (GTK_OBJECT
+		      (vwindow_button), "clicked",
+		      GTK_SIGNAL_FUNC (window_button_press), &options);
+  gtk_signal_connect (GTK_OBJECT
+		      (vroot_button), "clicked",
+		      GTK_SIGNAL_FUNC (desktop_button_press), &options);
+
+#ifdef FIXME
+  GTK_WIDGET_UNSET_FLAGS (hwindow_button, GTK_CAN_DEFAULT);
+  GTK_WIDGET_UNSET_FLAGS (hroot_button, GTK_CAN_DEFAULT);
+  GTK_WIDGET_UNSET_FLAGS (hwindow_button, GTK_CAN_FOCUS);
+  GTK_WIDGET_UNSET_FLAGS (hroot_button, GTK_CAN_FOCUS);
+  GTK_WIDGET_UNSET_FLAGS (vwindow_button, GTK_CAN_DEFAULT);
+  GTK_WIDGET_UNSET_FLAGS (vroot_button, GTK_CAN_DEFAULT);
+  GTK_WIDGET_UNSET_FLAGS (vwindow_button, GTK_CAN_FOCUS);
+  GTK_WIDGET_UNSET_FLAGS (vroot_button, GTK_CAN_FOCUS);
+#endif
+
+  set_tooltip (hwindow_button,
+	       (_ ("Grab a shot of a specific window or area")));
+  set_tooltip (hroot_button, (_ ("Grab a shot of your entire desktop")));
+  set_tooltip (vwindow_button,
+	       (_ ("Grab a shot of a specific window or area")));
+  set_tooltip (vroot_button, (_ ("Grab a shot of your entire desktop")));
+
+  vwindowpixmap = gnome_pixmap_new_from_xpm_d (window_icon_xpm);
+  vrootpixmap = gnome_pixmap_new_from_xpm_d (root_icon_xpm);
+  gtk_container_add (GTK_CONTAINER (vwindow_button), vwindowpixmap);
+  gtk_container_add (GTK_CONTAINER (vroot_button), vrootpixmap);
+
+  hwindowpixmap = gnome_pixmap_new_from_xpm_d (window_icon_xpm);
+  hrootpixmap = gnome_pixmap_new_from_xpm_d (root_icon_xpm);
+  gtk_container_add (GTK_CONTAINER (hwindow_button), hwindowpixmap);
+  gtk_container_add (GTK_CONTAINER (hroot_button), hrootpixmap);
+
+  gtk_widget_show (hwindowpixmap);
+  gtk_widget_show (vwindow_button);
+  gtk_widget_show (vroot_button);
+  gtk_widget_show (hwindow_button);
+  gtk_widget_show (hroot_button);
+  gtk_widget_show (hrootpixmap);
+  gtk_widget_show (vwindowpixmap);
+  gtk_widget_show (vrootpixmap);
+  gtk_widget_show (vbox);
+  gtk_widget_show (hbox);
+  gtk_widget_show (mainbox);
+	
+  gtk_container_add (GTK_CONTAINER (applet), mainbox);
+  
+  panel_applet_setup_menu (PANEL_APPLET (applet),
+			   test_applet_menu_xml,
+			   test_applet_menu_verbs,
+			   NULL);
+
+
+  gtk_widget_show_all (GTK_WIDGET (applet));
+  
+  g_signal_connect (G_OBJECT (applet), "change_size", 
+  		    G_CALLBACK (applet_change_pixel_size), NULL);
+  		 
+  g_signal_connect (G_OBJECT (applet), "change_orient",
+  		    G_CALLBACK (cb_applet_change_orient), NULL);
+		  
+  return TRUE;
+}
+
+static gboolean
+screenshooter_applet_factory (PanelApplet *applet,
+			      const gchar *iid,
+			      gpointer     data)
+{
+	gboolean retval = FALSE;
+    
+	if (!strcmp (iid, "OAFIID:GNOME_ScreenshooterApplet"))
+		retval = screenshooter_applet_fill (applet); 
+    
+	return retval;
+}
+
+PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_ScreenshooterApplet_Factory",
+			     PANEL_TYPE_APPLET,
+			     "Screenshooter applet",
+			     "0",
+			     screenshooter_applet_factory,
+			     NULL)
+
